@@ -10,9 +10,9 @@ use Fcntl ':mode';
 use Perinci::Sub::Util qw(gen_modified_sub);
 
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
-our $DATE = '2023-10-28'; # DATE
+our $DATE = '2023-10-29'; # DATE
 our $DIST = 'App-FileSortUtils'; # DIST
-our $VERSION = '0.001'; # VERSION
+our $VERSION = '0.002'; # VERSION
 
 our %SPEC;
 
@@ -54,12 +54,12 @@ our %argspecs_common = (
         schema => 'uint*',
         description => <<'MARKDOWN',
 
-Difference between `num_results` and `num_ranks`: `num_results` specifies number
-of results regardless of ranks while `num_ranks` (`-n` option) returns number of
-ranks. For example, if sorting is by reverse size and if `num_results` is set to
-1 and there are 2 files with the same largest size then only 1 of those files
-will be returned. With `num_ranks` set to 1, both files will be returned because
-are they both rank #1.
+Difference between `num_results` and `num_ranks`: `num_results` (`-n` option)
+specifies number of results regardless of ranks while `num_ranks` (`-N` option)
+returns number of ranks. For example, if sorting is by reverse size and if
+`num_results` is set to 1 and there are 2 files with the same largest size then
+only 1 of those files will be returned. With `num_ranks` set to 1, both files
+will be returned because are they both rank #1.
 
 MARKDOWN
         cmdline_aliases => {N=>{}},
@@ -120,6 +120,14 @@ MARKDOWN
     },
 );
 
+our %argspec_ignore_case = (
+    ignore_case => {
+        schema => 'bool*',
+        cmdline_aliases => {i=>{}},
+        tags => ['category:sorting'],
+    },
+);
+
 $SPEC{sort_files} = {
     v => 1.1,
     summary => 'Sort files in a directory and display the result in a flexible way',
@@ -220,43 +228,58 @@ sub sort_files {
         return [400, "Please specify one of by_field/by_sortsub/by_code"];
     } # SET_CODE_CMP
 
+    my (@ranks, @sorted_files);
   SORT: {
-        @files = sort {
-            ;
-            my ($key1, $key2);
-            {
-                local $_ = $a;
-                $key1 = $code_key->($a);
-            }
-            {
-                local $_ = $b;
-                $key2 = $code_key->($b);
-            }
-            $args{reverse} ? $code_cmp->($key2, $key1) : $code_cmp->($key1, $key2);
-        } @files;
+        require List::Rank;
+
+        my @res = List::Rank::sortrankby(
+            sub {
+                ;
+                my ($key1, $key2);
+                {
+                    local $_ = $a;
+                    $key1 = $code_key->($a);
+                }
+                {
+                    local $_ = $b;
+                    $key2 = $code_key->($b);
+                }
+                $args{reverse} ? $code_cmp->($key2, $key1) : $code_cmp->($key1, $key2);
+            },
+            @files
+        );
+        while (my ($e1, $e2) = splice @res, 0, 2) {
+            push @sorted_files, $e1;
+            push @ranks, $e2;
+        }
     } # SORT
 
   NUM_RESULTS: {
         last unless defined $args{num_results} && $args{num_results} > 0;
-        last unless $args{num_results} < @files;
-        splice @files, $args{num_results};
+        last unless $args{num_results} < @sorted_files;
+        splice @sorted_files, $args{num_results};
     }
 
   NUM_RANKS: {
         last unless defined $args{num_ranks} && $args{num_ranks} > 0;
-
-        return [501, "Not yet implemented"];
-        #require List::Rank;
-        #my @filenames = map { $_->{name} } @files;
-        #my @ranks = List::Rank::rankstr(
-        #splice @files, $args{num_results};
+        my @res;
+        my $i = -1;
+        while (1) {
+            $i++;
+            last if $i >= @sorted_files;
+            my $rank = $ranks[$i];
+            $rank =~ s/=\z//;
+            last if $rank > $args{num_ranks};
+            push @res, $sorted_files[$i];
+        }
+        @sorted_files = @res;
     }
 
     unless ($args{detail}) {
-        @files = map { $_->{name} } @files;
+        @sorted_files = map { $_->{name} } @sorted_files;
     }
 
-    [200, "OK", \@files];
+    [200, "OK", \@sorted_files];
 }
 
 # foremost
@@ -265,6 +288,9 @@ gen_modified_sub(
     output_name => 'foremost',
     base_name => 'sort_files',
     remove_args => [keys %argspecs_sort],
+    add_args => {
+        %argspec_ignore_case,
+    },
     summary => 'Return file(s) which are alphabetically the first',
     description => <<'MARKDOWN',
 
@@ -284,6 +310,7 @@ MARKDOWN
         sort_files(
             %args,
             by_field => 'name',
+            ($args{ignore_case} ? (key=>sub {lc $_[0]{name}}) : ()),
         );
     },
 );
@@ -294,6 +321,9 @@ gen_modified_sub(
     output_name => 'hindmost',
     base_name => 'sort_files',
     remove_args => [keys %argspecs_sort],
+    add_args => {
+        %argspec_ignore_case,
+    },
     summary => 'Return file(s) which are alphabetically the last',
     description => <<'MARKDOWN',
 
@@ -313,6 +343,7 @@ MARKDOWN
         sort_files(
             %args,
             by_field => 'name',
+            ($args{ignore_case} ? (key=>sub {lc $_[0]{name}}) : ()),
             reverse => 1,
         );
     },
@@ -395,6 +426,16 @@ gen_modified_sub(
     summary => 'Return the newest file(s) in a directory',
     description => <<'MARKDOWN',
 
+Suppose a new file is downloaded in `~/Downloads`, but you are not sure of its
+name. You just want to move that file, which you are pretty sure is the newest
+in the `Downloads` directory, somewhere else. So from the CLI in `~/Downloads`:
+
+    % mv `newest -f` /somewhere/else
+
+or, from `/somewhere/else`:
+
+    % mv `newest -f ~/Downloads` .
+
 File is deemed as newest by its mtime.
 
 Some examples:
@@ -457,6 +498,72 @@ MARKDOWN
     },
 );
 
+# longest_name
+gen_modified_sub(
+    die => 1,
+    output_name => 'longest_name',
+    base_name => 'sort_files',
+    remove_args => [keys %argspecs_sort],
+    summary => 'Return file(s) with the longest name in a directory',
+    description => <<'MARKDOWN',
+
+Some examples:
+
+    # return file with the longest name in current directory
+    % longest-name -f
+
+    # return file(s) with the longest name in /some/dir. if there are multiple
+    # files with the same length, they will all be returned.
+    % longest-name -N1 -f /some/dir
+
+MARKDOWN
+    output_code => sub {
+        my %args = @_;
+
+        unless (defined $args{num_results} || defined $args{num_ranks}) {
+            $args{num_results} = 1;
+        }
+
+        sort_files(
+            %args,
+            by_code => sub { length($_[1]{name}) <=> length($_[0]{name}) },
+        );
+    },
+);
+
+# shortest_name
+gen_modified_sub(
+    die => 1,
+    output_name => 'shortest_name',
+    base_name => 'sort_files',
+    remove_args => [keys %argspecs_sort],
+    summary => 'Return file(s) with the shortest name in a directory',
+    description => <<'MARKDOWN',
+
+Some examples:
+
+    # return file with the shortest name in current directory
+    % shortest-name -f
+
+    # return file(s) with the shortest name in /some/dir. if there are multiple
+    # files with the same length, they will all be returned.
+    % shortest-name -N1 -f /some/dir
+
+MARKDOWN
+    output_code => sub {
+        my %args = @_;
+
+        unless (defined $args{num_results} || defined $args{num_ranks}) {
+            $args{num_results} = 1;
+        }
+
+        sort_files(
+            %args,
+            by_code => sub { length($_[0]{name}) <=> length($_[1]{name}) },
+        );
+    },
+);
+
 1;
 # ABSTRACT: Utilities related to sorting files in a directory
 
@@ -472,7 +579,7 @@ App::FileSortUtils - Utilities related to sorting files in a directory
 
 =head1 VERSION
 
-This document describes version 0.001 of App::FileSortUtils (from Perl distribution App-FileSortUtils), released on 2023-10-28.
+This document describes version 0.002 of App::FileSortUtils (from Perl distribution App-FileSortUtils), released on 2023-10-29.
 
 =head1 DESCRIPTION
 
@@ -486,13 +593,17 @@ This distribution provides the following command-line utilities:
 
 =item 3. L<largest>
 
-=item 4. L<newest>
+=item 4. L<longest-name>
 
-=item 5. L<oldest>
+=item 5. L<newest>
 
-=item 6. L<smallest>
+=item 6. L<oldest>
 
-=item 7. L<sort-files>
+=item 7. L<shortest-name>
+
+=item 8. L<smallest>
+
+=item 9. L<sort-files>
 
 =back
 
@@ -526,16 +637,20 @@ Arguments ('*' denotes required arguments):
 
 Directory to sort files of, defaults to current directory.
 
+=item * B<ignore_case> => I<bool>
+
+(No description)
+
 =item * B<num_ranks> => I<uint>
 
 Number of ranks to return.
 
-Difference between C<num_results> and C<num_ranks>: C<num_results> specifies number
-of results regardless of ranks while C<num_ranks> (C<-n> option) returns number of
-ranks. For example, if sorting is by reverse size and if C<num_results> is set to
-1 and there are 2 files with the same largest size then only 1 of those files
-will be returned. With C<num_ranks> set to 1, both files will be returned because
-are they both rank #1.
+Difference between C<num_results> and C<num_ranks>: C<num_results> (C<-n> option)
+specifies number of results regardless of ranks while C<num_ranks> (C<-N> option)
+returns number of ranks. For example, if sorting is by reverse size and if
+C<num_results> is set to 1 and there are 2 files with the same largest size then
+only 1 of those files will be returned. With C<num_ranks> set to 1, both files
+will be returned because are they both rank #1.
 
 =item * B<num_results> => I<uint>
 
@@ -588,16 +703,20 @@ Arguments ('*' denotes required arguments):
 
 Directory to sort files of, defaults to current directory.
 
+=item * B<ignore_case> => I<bool>
+
+(No description)
+
 =item * B<num_ranks> => I<uint>
 
 Number of ranks to return.
 
-Difference between C<num_results> and C<num_ranks>: C<num_results> specifies number
-of results regardless of ranks while C<num_ranks> (C<-n> option) returns number of
-ranks. For example, if sorting is by reverse size and if C<num_results> is set to
-1 and there are 2 files with the same largest size then only 1 of those files
-will be returned. With C<num_ranks> set to 1, both files will be returned because
-are they both rank #1.
+Difference between C<num_results> and C<num_ranks>: C<num_results> (C<-n> option)
+specifies number of results regardless of ranks while C<num_ranks> (C<-N> option)
+returns number of ranks. For example, if sorting is by reverse size and if
+C<num_results> is set to 1 and there are 2 files with the same largest size then
+only 1 of those files will be returned. With C<num_ranks> set to 1, both files
+will be returned because are they both rank #1.
 
 =item * B<num_results> => I<uint>
 
@@ -658,12 +777,78 @@ Directory to sort files of, defaults to current directory.
 
 Number of ranks to return.
 
-Difference between C<num_results> and C<num_ranks>: C<num_results> specifies number
-of results regardless of ranks while C<num_ranks> (C<-n> option) returns number of
-ranks. For example, if sorting is by reverse size and if C<num_results> is set to
-1 and there are 2 files with the same largest size then only 1 of those files
-will be returned. With C<num_ranks> set to 1, both files will be returned because
-are they both rank #1.
+Difference between C<num_results> and C<num_ranks>: C<num_results> (C<-n> option)
+specifies number of results regardless of ranks while C<num_ranks> (C<-N> option)
+returns number of ranks. For example, if sorting is by reverse size and if
+C<num_results> is set to 1 and there are 2 files with the same largest size then
+only 1 of those files will be returned. With C<num_ranks> set to 1, both files
+will be returned because are they both rank #1.
+
+=item * B<num_results> => I<uint>
+
+Number of results to return.
+
+=item * B<type> => I<str>
+
+Only include files of certain type.
+
+
+=back
+
+Returns an enveloped result (an array).
+
+First element ($status_code) is an integer containing HTTP-like status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+($reason) is a string containing error message, or something like "OK" if status is
+200. Third element ($payload) is the actual result, but usually not present when enveloped result is an error response ($status_code is not 2xx). Fourth
+element (%result_meta) is called result metadata and is optional, a hash
+that contains extra information, much like how HTTP response headers provide additional metadata.
+
+Return value:  (any)
+
+
+
+=head2 longest_name
+
+Usage:
+
+ longest_name(%args) -> [$status_code, $reason, $payload, \%result_meta]
+
+Return file(s) with the longest name in a directory.
+
+Some examples:
+
+ # return file with the longest name in current directory
+ % longest-name -f
+ 
+ # return file(s) with the longest name in /some/dir. if there are multiple
+ # files with the same length, they will all be returned.
+ % longest-name -N1 -f /some/dir
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<detail> => I<true>
+
+(No description)
+
+=item * B<dir> => I<dirname::default_curdir>
+
+Directory to sort files of, defaults to current directory.
+
+=item * B<num_ranks> => I<uint>
+
+Number of ranks to return.
+
+Difference between C<num_results> and C<num_ranks>: C<num_results> (C<-n> option)
+specifies number of results regardless of ranks while C<num_ranks> (C<-N> option)
+returns number of ranks. For example, if sorting is by reverse size and if
+C<num_results> is set to 1 and there are 2 files with the same largest size then
+only 1 of those files will be returned. With C<num_ranks> set to 1, both files
+will be returned because are they both rank #1.
 
 =item * B<num_results> => I<uint>
 
@@ -697,6 +882,16 @@ Usage:
 
 Return the newest file(s) in a directory.
 
+Suppose a new file is downloaded in C<~/Downloads>, but you are not sure of its
+name. You just want to move that file, which you are pretty sure is the newest
+in the C<Downloads> directory, somewhere else. So from the CLI in C<~/Downloads>:
+
+ % mv C<newest -f> /somewhere/else
+
+or, from C</somewhere/else>:
+
+ % mv C<newest -f ~/Downloads> .
+
 File is deemed as newest by its mtime.
 
 Some examples:
@@ -726,12 +921,12 @@ Directory to sort files of, defaults to current directory.
 
 Number of ranks to return.
 
-Difference between C<num_results> and C<num_ranks>: C<num_results> specifies number
-of results regardless of ranks while C<num_ranks> (C<-n> option) returns number of
-ranks. For example, if sorting is by reverse size and if C<num_results> is set to
-1 and there are 2 files with the same largest size then only 1 of those files
-will be returned. With C<num_ranks> set to 1, both files will be returned because
-are they both rank #1.
+Difference between C<num_results> and C<num_ranks>: C<num_results> (C<-n> option)
+specifies number of results regardless of ranks while C<num_ranks> (C<-N> option)
+returns number of ranks. For example, if sorting is by reverse size and if
+C<num_results> is set to 1 and there are 2 files with the same largest size then
+only 1 of those files will be returned. With C<num_ranks> set to 1, both files
+will be returned because are they both rank #1.
 
 =item * B<num_results> => I<uint>
 
@@ -794,12 +989,78 @@ Directory to sort files of, defaults to current directory.
 
 Number of ranks to return.
 
-Difference between C<num_results> and C<num_ranks>: C<num_results> specifies number
-of results regardless of ranks while C<num_ranks> (C<-n> option) returns number of
-ranks. For example, if sorting is by reverse size and if C<num_results> is set to
-1 and there are 2 files with the same largest size then only 1 of those files
-will be returned. With C<num_ranks> set to 1, both files will be returned because
-are they both rank #1.
+Difference between C<num_results> and C<num_ranks>: C<num_results> (C<-n> option)
+specifies number of results regardless of ranks while C<num_ranks> (C<-N> option)
+returns number of ranks. For example, if sorting is by reverse size and if
+C<num_results> is set to 1 and there are 2 files with the same largest size then
+only 1 of those files will be returned. With C<num_ranks> set to 1, both files
+will be returned because are they both rank #1.
+
+=item * B<num_results> => I<uint>
+
+Number of results to return.
+
+=item * B<type> => I<str>
+
+Only include files of certain type.
+
+
+=back
+
+Returns an enveloped result (an array).
+
+First element ($status_code) is an integer containing HTTP-like status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+($reason) is a string containing error message, or something like "OK" if status is
+200. Third element ($payload) is the actual result, but usually not present when enveloped result is an error response ($status_code is not 2xx). Fourth
+element (%result_meta) is called result metadata and is optional, a hash
+that contains extra information, much like how HTTP response headers provide additional metadata.
+
+Return value:  (any)
+
+
+
+=head2 shortest_name
+
+Usage:
+
+ shortest_name(%args) -> [$status_code, $reason, $payload, \%result_meta]
+
+Return file(s) with the shortest name in a directory.
+
+Some examples:
+
+ # return file with the shortest name in current directory
+ % shortest-name -f
+ 
+ # return file(s) with the shortest name in /some/dir. if there are multiple
+ # files with the same length, they will all be returned.
+ % shortest-name -N1 -f /some/dir
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<detail> => I<true>
+
+(No description)
+
+=item * B<dir> => I<dirname::default_curdir>
+
+Directory to sort files of, defaults to current directory.
+
+=item * B<num_ranks> => I<uint>
+
+Number of ranks to return.
+
+Difference between C<num_results> and C<num_ranks>: C<num_results> (C<-n> option)
+specifies number of results regardless of ranks while C<num_ranks> (C<-N> option)
+returns number of ranks. For example, if sorting is by reverse size and if
+C<num_results> is set to 1 and there are 2 files with the same largest size then
+only 1 of those files will be returned. With C<num_ranks> set to 1, both files
+will be returned because are they both rank #1.
 
 =item * B<num_results> => I<uint>
 
@@ -860,12 +1121,12 @@ Directory to sort files of, defaults to current directory.
 
 Number of ranks to return.
 
-Difference between C<num_results> and C<num_ranks>: C<num_results> specifies number
-of results regardless of ranks while C<num_ranks> (C<-n> option) returns number of
-ranks. For example, if sorting is by reverse size and if C<num_results> is set to
-1 and there are 2 files with the same largest size then only 1 of those files
-will be returned. With C<num_ranks> set to 1, both files will be returned because
-are they both rank #1.
+Difference between C<num_results> and C<num_ranks>: C<num_results> (C<-n> option)
+specifies number of results regardless of ranks while C<num_ranks> (C<-N> option)
+returns number of ranks. For example, if sorting is by reverse size and if
+C<num_results> is set to 1 and there are 2 files with the same largest size then
+only 1 of those files will be returned. With C<num_ranks> set to 1, both files
+will be returned because are they both rank #1.
 
 =item * B<num_results> => I<uint>
 
@@ -950,12 +1211,12 @@ Another example, to generate length of name as key:
 
 Number of ranks to return.
 
-Difference between C<num_results> and C<num_ranks>: C<num_results> specifies number
-of results regardless of ranks while C<num_ranks> (C<-n> option) returns number of
-ranks. For example, if sorting is by reverse size and if C<num_results> is set to
-1 and there are 2 files with the same largest size then only 1 of those files
-will be returned. With C<num_ranks> set to 1, both files will be returned because
-are they both rank #1.
+Difference between C<num_results> and C<num_ranks>: C<num_results> (C<-n> option)
+specifies number of results regardless of ranks while C<num_ranks> (C<-N> option)
+returns number of ranks. For example, if sorting is by reverse size and if
+C<num_results> is set to 1 and there are 2 files with the same largest size then
+only 1 of those files will be returned. With C<num_ranks> set to 1, both files
+will be returned because are they both rank #1.
 
 =item * B<num_results> => I<uint>
 
